@@ -38,6 +38,7 @@ SERIES_PATH = 'series.conf'
 
 MAND_KEYS = ('Name',
              'Summary',
+             'Description',
              'Version',
              'Group',
              'License',
@@ -45,6 +46,7 @@ MAND_KEYS = ('Name',
 
 SUB_MAND_KEYS = ('Name',
                  'Summary',
+                 'Description',
                  'Group',
                 )
 
@@ -60,6 +62,7 @@ BOOLNO_KEYS = ('Check',
                'NoFiles',
                'NoDesktop',
                'UpdateDesktopDB',
+               'NoIconCache',
               )
 # boolean keys with the default 'True' value
 BOOLYES_KEYS = ('UseAsNeeded',
@@ -140,6 +143,7 @@ SUBAVAIL_KEYS = ('Name',
                  'NoAutoReq',
                  'NoAutoProv',
                  'NoAutoReqProv',
+                 'NoIconCache',
                  'FilesInput',
                  'Version', 'Release', 'Epoch', 'URL', 'BuildArch' # very rare
                 )
@@ -163,11 +167,29 @@ ARCHED_KEYS = ('Requires',
                'ConfigOptions',
                'QMakeOptions',
               )
-ARCHS = ('ix86', 'arm')
+ARCHS = {'ix86': '%{ix86}',
+         'arm': '%{arm}',
+         'armv5': 'armv5el armv5tel armv5tejl',
+         'armv7': 'armv7el armv7tel armv7l armv7hl armv7nhl',
+        }
 
 CONFIGURES = ('configure', 'reconfigure', 'autogen', 'cmake', 'none')
 BUILDERS = ('make', 'single-make', 'python', 'perl', 'qmake', 'cmake', 'none')
 
+# global helper functions
+def arch_split(value):
+    m = re.match('^(\w+):([^:]+)', value)
+    if m:
+        arch = m.group(1)
+        left = m.group(2)
+        if arch in ARCHS:
+            return arch, ARCHS[arch], left
+        else:
+            return arch, arch, left
+
+    else:
+        return '', '', value
+
 class GitAccess():
     def __init__(self, path):
         self.path = path
@@ -233,6 +255,7 @@ class RPMWriter():
         self.clean_old = clean_old
         self.download_new = download_new
         self.skip_scm = skip_scm
+        self.uiwidget = None # default as gtk2 if Qt not detected
 
         # initialize extra info for spec
         self.extra = { 'subpkgs': {}, 'content': {} }
@@ -370,7 +393,7 @@ class RPMWriter():
 
             return keys
 
-        def _check_mandatory_keys(metadata, subpkg = None):
+        def _check_mandatory_keys(metadata, subpkg = False):
             """ return [] if all mandatory keys found, otherwise return the lost keys """
             if subpkg:
                 mkeys = list(SUB_MAND_KEYS)
@@ -458,9 +481,10 @@ class RPMWriter():
 
         def _check_key_desc(metadata):
             """ sub-routine for 'description' checking """
-            if 'Description' not in metadata or \
-                metadata['Description'] == '%{summary}':
+            if metadata['Description'] == '%{summary}' or \
+               metadata['Description'] == metadata['Summary']:
                 return False
+
             return True
 
         def _check_listkey(metadata, key):
@@ -484,18 +508,28 @@ class RPMWriter():
         def _check_arched_keys(metadata):
             """ sub-routine for ARCH namespace available keys """
             def _check_arch(key, item):
-                if re.match('^\w+:[^:]+', item):
-                    arch = item.split(':')[0].strip()
-                    if arch not in ARCHS:
-                        logger.warning('unsupport arch namespace: %s in key %s' % (arch, key))
+                if isinstance(item, dict):
+                    logger.warning('For arch prefixed %s value "%s", please do NOT leave extra spaces after ":", skipped!' % \
+                                    (key, ':'.join(map(str, item.popitem()))))
+                    return False
+
+                arch = arch_split(item)[0]
+                if arch and arch not in ARCHS:
+                    logger.warning('unsupport arch namespace: %s in key %s' % (arch, key))
+
+                return True
 
             for key in ARCHED_KEYS:
                 if key in metadata:
                     if key in STR_KEYS:
-                        _check_arch(key, metadata[key])
+                        if not _check_arch(key, metadata[key]):
+                            del metadata[key]
                     elif key in LIST_KEYS:
                         for item in metadata[key]:
-                            _check_arch(key, item)
+                            if not _check_arch(key, item):
+                                metadata[key].remove(item)
+                                if not metadata[key]:
+                                    del metadata[key]
 
         def _check_key_localename(metadata):
             """ sub-routine for 'LocaleName' checking """
@@ -570,9 +604,12 @@ class RPMWriter():
             logger.error('Missing mandatory keys for main package: %s' % ', '.join(keys))
         if "SubPackages" in self.metadata:
             for sp in self.metadata["SubPackages"]:
-                keys = _check_mandatory_keys(sp, sp['Name'])
+                keys = _check_mandatory_keys(sp, True)
                 if keys:
-                    logger.error('Missing mandatory keys for sub-pkg %s: %s' % (sp['Name'], ', '.join(keys)))
+                    if 'Name' in keys:
+                        logger.error('Missing mandatory keys for sub-pkg: Name')
+                    else:
+                        logger.error('Missing mandatory keys for sub-pkg "%s": %s' % (sp['Name'], ', '.join(keys)))
 
         # checking for unexpected keys
         keys = _check_invalid_keys(self.metadata)
@@ -694,11 +731,14 @@ PkgBR:
 
         # checking for validation of 'Description'
         if not _check_key_desc(self.metadata):
-            logger.warning('main package has no qualified "Description" tag')
+            logger.error('main package has no qualified "Description" tag\n\
+\t"Description" cannot be "%{summary}" or equal to the value of "Summary"')
         if "SubPackages" in self.metadata:
             for sp in self.metadata["SubPackages"]:
                 if not _check_key_desc(sp):
-                    logger.warning('sub-pkg: %s has no qualified "Description" tag' % sp['Name'])
+                    logger.error('sub-pkg: "%s" has no qualified "Description" tag\n\
+\t"Description" cannot be "%%{summary}" or equal to the value of "Summary"'
+                    % sp['Name'])
 
         # checking for validation of 'LocaleName' and 'LocaleOptions'
         if not _check_key_localename(self.metadata):
@@ -832,7 +872,7 @@ PkgBR:
             fpath = os.path.basename(uri)
             fpath = fpath.replace('%{name}', self.pkg)
             fpath = fpath.replace('%{version}', self.version)
-            if os.path.exists(fpath):
+            if os.path.exists(fpath) and os.path.getsize(fpath):
                 try:
                     if tarfile.is_tarfile(fpath):
                         tarball = fpath
@@ -865,16 +905,24 @@ PkgBR:
             # setting the default value firstly
             self.metadata['SourcePrefix'] = '%{name}-%{version}'
             if not prefix or prefix == '.':
+                # guess prefix from filename
                 if tarball:
-                    prefix, ignore = os.path.basename(tarball).split('.tar.')
+                    if '.tar.' in tarball:
+                        prefix = os.path.basename(tarball).split('.tar.')[0]
+                    else:
+                        # strip the ext name
+                        prefix = os.path.splitext(tarball)[0]
 
             if prefix and prefix != '%s-%s' % (self.pkg, self.version):
                 prefix = prefix.replace(self.pkg, '%{name}')
                 prefix = prefix.replace(self.version, '%{version}')
                 self.metadata['SourcePrefix'] = prefix
 
-    def _parse_series(self, patches, comments):
-        comment = ""
+    def _parse_series(self):
+        patches = []
+        comments = []
+
+        comment = ''
         for line in file(SERIES_PATH):
             if not line.strip():
                 continue
@@ -883,9 +931,11 @@ PkgBR:
             else:
                 line = line.strip()
                 patches.append(line)
-                comments.append(comment + '# ' + line)
+                comments.append(comment.rstrip())
                 comment = ''
 
+        return patches, comments
+
     def _cleanup_boolkeys(self, items):
         """ clean up all boolean type keys,
             use the exists status to present bool value
@@ -955,6 +1005,10 @@ PkgBR:
             self.metadata.update(yaml.load(self.stream))
         except yaml.scanner.ScannerError, e:
             logger.error('syntax error found in yaml: %s' % str(e))
+        except yaml.parser.ParserError, e:
+            logger.error('syntax error found in yaml: %s' % str(e))
+        except ValueError:
+            logger.error('Please check if the input file is in YAML format')
         except TypeError:
             # empty can lead here
             logger.error('Empty yaml file: %s' % self.yaml_fpath)
@@ -1030,10 +1084,7 @@ PkgBR:
 
             self.metadata['Patches']   = []
             self.metadata['PatchOpts'] = []
-            self.metadata['PatchCmts'] = []
             for patch in patches:
-                self.metadata['PatchCmts'].append('# ' + patch)
-
                 if isinstance(patch, str):
                     self.metadata['Patches'].append(patch)
                     self.metadata['PatchOpts'].append('-p1')
@@ -1049,10 +1100,7 @@ PkgBR:
             if "Patches" in self.metadata:
                 logger.warning('Both "Patches" tag in yaml and series.conf exists, please use only one.')
             else:
-                self.metadata['Patches'] = []
-                self.metadata['PatchCmts'] = []
-                self._parse_series(self.metadata['Patches'],
-                                    self.metadata['PatchCmts'])
+                self.metadata['Patches'], self.metadata['PatchCmts'] = self._parse_series()
 
         if 'Sources' in self.metadata:
             self._analyze_source()
@@ -1089,7 +1137,19 @@ PkgBR:
         if "SubPackages" in self.metadata:
             for sp in self.metadata["SubPackages"]:
                 self.extra['subpkgs'][sp['Name']] = copy.deepcopy(self.extra_per_pkg)
+        if "AutoSubPackages" in self.metadata:
+            for asp in self.metadata["AutoSubPackages"]:
+                self.extra['subpkgs'][asp] = copy.deepcopy(self.extra_per_pkg)
 
+        # detect the using UI widget, QT or Gtk2
+        all_pkgbr = []
+        if 'PkgBR' in self.metadata:
+            all_pkgbr += self.metadata['PkgBR']
+        if 'PkgConfigBR' in self.metadata:
+            all_pkgbr += self.metadata['PkgConfigBR']
+        for br in all_pkgbr:
+            if br.startswith('Qt'):
+                self.uiwidget = 'Qt'
 
         """ NOTE
         we need NOT to do the following checking:
@@ -1098,6 +1158,17 @@ PkgBR:
         They should be checked by users manually.
         """
 
+    def _lookup_pkgmeta(self, pkgname):
+        if pkgname == 'main':
+            return self.metadata
+
+        for sp in self.metadata['SubPackages']:
+            if sp['Name'] == pkgname:
+                return sp
+
+        # not found
+        return {}
+
     def parse_files(self, files = {}, docs = {}):
 
         py_path_check = False
@@ -1109,13 +1180,17 @@ PkgBR:
                 py_path = '%{python_sitearch}'
 
         for pkg_name,v in files.iteritems():
+            pkg_meta = self._lookup_pkgmeta(pkg_name)
+
             if pkg_name == 'main':
                 pkg_extra = self.extra
             else:
                 pkg_extra = self.extra['subpkgs'][pkg_name]
 
             for l in v:
-                if re.match('.*\.info\..*', l) or re.match('.*(usr/share/info|%{_infodir}).*info\..*$', l):
+                if re.match('\s*%exclude\s.*', l):
+                    pass # not match anyting excluded files
+                elif re.match('.*\.info\..*', l) or re.match('.*(usr/share/info|%{_infodir}).*info\..*$', l):
                     p1 = re.compile('^%doc\s+(.*)')
                     l1 = p1.sub(r'\1', l)
                     pkg_extra['Infos'].append(l1)
@@ -1123,7 +1198,11 @@ PkgBR:
                 elif re.match('.*(usr/share|%{_datadir})/applications/.*\.desktop$', l):
                     if 'NoDesktop' not in self.metadata:
                         pkg_extra['Desktop'] = True
-                elif re.match('.*etc/rc.d/init.d.*', l) or re.match('.*etc/init.d.*', l):
+                elif re.match('.*(/etc|%{_sysconfdir})/rc.d/init.d/.*', l) or \
+                     re.match('.*(/etc|%{_sysconfdir})/init.d/.*', l) or \
+                     re.match('.*%{_initddir}/.*', l) or \
+                     re.match('.*%{_initrddir}/.*', l):
+
                     pkg_extra['Service'] = True
                 elif re.match('.*(%{_libdir}|%{_lib}|/lib|/usr/lib)/[^/]*[.*?]+so([.*?]+.*$|$)', l) or \
                    re.match('.*(/ld.so.conf.d/).*', l):
@@ -1140,6 +1219,14 @@ PkgBR:
                     pkg_extra['Schema'] = True
                     pkg_extra['Schemas'].append(l)
                 elif re.match('.*\/icons\/.*', l):
+                    if 'NoIconCache' in pkg_meta and pkg_meta['NoIconCache'] == True:
+                        # using "NoIconCache" to avoid cache explicitly
+                        continue
+
+                    if self.uiwidget and self.uiwidget == 'Qt':
+                        # disable icon files handling for Qt based app
+                        continue
+
                     pkg_extra['Icon'] = True
 
                 # special checking for python packages
@@ -1223,6 +1310,10 @@ PkgBR:
                 record = False
                 if not recording: continue # empty
 
+                if matchout.group(2) and matchout.group(1) in ["files", "post","postun", "pre", "preun"]:
+                    if not matchout.group(2) in self.extra['subpkgs']:
+                        logger.error('In spec %s section for %s package. Package does not excist in YAML.' % (matchout.group(1), matchout.group(2)))
+
                 if matchout.group(1) == "files":
                     if matchout.group(2):
                         files[matchout.group(2)] = recording
@@ -1303,9 +1394,9 @@ PkgBR:
             self._check_dup_files(content['files'][key])
 
         # checking duplicate 'rm -rf %{buildroot}'
-        re_cleanup = re.compile('(rm|\%\{__rm\})\W+-rf\W+(\$RPM_BUILD_ROOT|\%\{buildroot\})')
+        re_cleanup = re.compile('^(?:rm|\%\{__rm\})\W+-rf\W+(?:\$RPM_BUILD_ROOT|\%\{buildroot\})/?$')
         if 'install' in content and 'post' in content['install']:
-            if re_cleanup.search(content['install']['post'][0]):
+            if re_cleanup.match(content['install']['post'][0]):
                 logger.warning('duplicate buildroot cleanup found in the first line of install_post, remove it')
 
         return content
@@ -1351,37 +1442,34 @@ PkgBR:
                 self.extra['content'].update({'files': {'main': self.extras_filelist}})
         """
 
-        try:
-            # TODO, cleanup docs handling when all pkgs need not, include spec.tmpl
-            docs = {}
-            if 'Documents' in self.metadata:
-                docs['main'] = self.metadata['Documents']
-            if "SubPackages" in self.metadata:
-                for sp in self.metadata["SubPackages"]:
-                    if 'Documents' in sp:
-                        docs[sp['Name']] = sp['Documents']
+        # TODO, cleanup docs handling when all pkgs need not, include spec.tmpl
+        docs = {}
+        if 'Documents' in self.metadata:
+            docs['main'] = self.metadata['Documents']
+        if "SubPackages" in self.metadata:
+            for sp in self.metadata["SubPackages"]:
+                if 'Documents' in sp:
+                    docs[sp['Name']] = sp['Documents']
 
-            if 'files' in self.extra['content']:
-                files = copy.deepcopy(self.extra['content']['files'])
+        if 'files' in self.extra['content']:
+            files = copy.deepcopy(self.extra['content']['files'])
+        else:
+            files = {}
+
+        if 'Files' in self.metadata:
+            if 'main' in files:
+                files['main'] += self.metadata['Files']
             else:
-                files = {}
+                files['main'] = self.metadata['Files']
+        if "SubPackages" in self.metadata:
+            for sp in self.metadata["SubPackages"]:
+                if 'Files' in sp:
+                    if sp['Name'] in files:
+                        files[sp['Name']] += sp['Files']
+                    else:
+                        files[sp['Name']] = sp['Files']
 
-            if 'Files' in self.metadata:
-                if 'main' in files:
-                    files['main'] += self.metadata['Files']
-                else:
-                    files['main'] = self.metadata['Files']
-            if "SubPackages" in self.metadata:
-                for sp in self.metadata["SubPackages"]:
-                    if 'Files' in sp:
-                        if sp['Name'] in files:
-                            files[sp['Name']] += sp['Files']
-                        else:
-                            files[sp['Name']] = sp['Files']
-
-            self.parse_files(files, docs)
-        except KeyError:
-            pass
+        self.parse_files(files, docs)
 
         # adding automatic requires according %files
         self._gen_auto_requires(self.metadata, self.extra)
@@ -1407,7 +1495,8 @@ PkgBR:
 
         spec_content = spec.spec(searchList=[{
                                         'metadata': self.metadata,
-                                        'extra': self.extra
+                                        'extra': self.extra,
+                                        'arch_split': arch_split
                                       }]).respond()
 
         file = open(specfile, "w")
